 /*
 *  file      : cd_cddb.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <glib/gprintf.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <cddb/cddb.h>
#include <sys/types.h>
#include <cdio/cdio.h>
#include <cdio/cdtext.h>
#include <pthread.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoomc/lib.h"
#endif

#include "global.h"
#include "cd_cddb.h"
#include "get_info.h"
#include "configuser.h"
#include "scan.h"
#include "win_scan.h"
#include "statusbar.h"
#include "poche.h"


#define CDDA_DATA_SIZE 2352



typedef struct {
	gint		Choice;
	gchar		*NameCD_Device;
	gboolean	bool_thread_read_toc_cd;
	gboolean	bool_timeout_read_toc_cd;
	guint		handler_timeout_read_toc_cd;
	gboolean	Bool_Read_Infos_cd;
} VAR_CD;

VAR_CD VarCD = {
		0,				// Choice
		NULL,				// *NameCD_Device
		FALSE,				// bool_thread_read_toc_cd
		FALSE,				// bool_timeout_read_toc_cd
		0,				// handler_timeout_read_toc_cd
		FALSE				// Bool_Read_Infos_cd
		};


typedef struct {
	gchar		*Duree;
	gchar		*Artist;
	gchar		*Title;
	gint		Sector;
	gint		SectorCdParanoia;
} VAR_STRUCT_CDDB;

typedef struct {
	gint		TotalTracks;
	gchar		Message [ 100 ];
	gchar		*TitleCD;
	gchar		*Title;
	gchar		*Artiste;
	gchar		*StrDureeCd;
	gint		NumYear;
	gchar		*StrYear;
	gboolean	BoolMultiArtiste;
	VAR_STRUCT_CDDB	*VarStructCddb;		// 0 .. TotalTracks -1
	
} VAR_CDDB;

VAR_CDDB VarCddb = {
		0,				// TotalTracks
		"",				// Message [ ]
		NULL,				// *TitleCD
		NULL,				// *Title
		NULL,				// *Artiste
		NULL,				// *StrDureeCd
		0,				// NumYear
		NULL,				// *StrYear
		FALSE				// BoolMultiArtiste
		};




gboolean bool_cancel_cddb = FALSE;	/* FALSE = continue ELSE STOP */





// Si 'FLAG == FALSE', force l'arret de la recherche sur le serveur freedb
//
void CdCddb_set_bool_cancel_cddb (gboolean FLAG)
{
	/*PRINT_FUNC_LF();*/
	bool_cancel_cddb = FLAG;
}
// 
// 
gboolean CdCddb_get_bool_cancel_cddb (void)
{
	return (bool_cancel_cddb);
}
//
//
gboolean CdCddb_test_access_web (void)
{
	// PRINT_FUNC_LF();
	
	if (libutils_find_file ("wget") == FALSE) {
		g_print ("WGET absent !\n");
		return (FALSE);
	}
	return (GetInfo_wget ());
}
// 
// 
void CdCddb_remove_struct( void )
{
	gint	NumList;
	
	VarCddb . TotalTracks = 0;
	VarCddb . Message [ 0 ] = '\0';
	if( NULL != VarCddb . TitleCD )		{ g_free( VarCddb . TitleCD );		VarCddb . TitleCD = NULL; }
	if( NULL != VarCddb . Title )		{ g_free( VarCddb . Title );		VarCddb . Title = NULL; }
	if( NULL != VarCddb . Artiste )		{ g_free( VarCddb . Artiste );		VarCddb . Artiste = NULL; }
	if( NULL != VarCddb . StrDureeCd )	{ g_free( VarCddb . StrDureeCd );	VarCddb . StrDureeCd = NULL; }
	// if( NULL != VarCddb . StrGenre )	{ g_free( VarCddb . StrGenre );		VarCddb . StrGenre = NULL; }
	if( NULL != VarCddb . StrYear )		{ g_free( VarCddb . StrYear );		VarCddb . StrYear = NULL; }
	
	if( NULL != VarCddb . VarStructCddb ) {
		for( NumList = 0; NumList < VarCddb . TotalTracks; NumList++ ) {
			
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Duree ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Duree );		VarCddb . VarStructCddb [ NumList ] . Duree = NULL;
			}
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Artist ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Artist );		VarCddb . VarStructCddb [ NumList ] . Artist = NULL;
			}
			if( NULL != VarCddb . VarStructCddb [ NumList ] . Title ) {
				g_free( VarCddb . VarStructCddb [ NumList ] . Title );		VarCddb . VarStructCddb [ NumList ] . Title = NULL;
			}
		}
		g_free( VarCddb . VarStructCddb );	VarCddb . VarStructCddb = NULL;
	}
}
// 
// 
gint CdCddb_cdparanoia_get_total_tracks( gchar *buffer )
{
	gchar  **Larrbuf = NULL;
	gint     Track;
	gint     TotalTracks = -1;
	gchar    *ptr;

	Larrbuf = g_strsplit( buffer, "\n", 0 );

	for( Track = 0; Larrbuf[ Track ]; Track++ ) {
		if( strstr (Larrbuf[ Track ], "TOTAL" )) {
			Track --;
			break;
		}
	}
	if( NULL != ( ptr = Larrbuf[ Track ] )) {
		while( *ptr == ' ' ) ptr ++;
		TotalTracks = atoi( ptr );
	}
	g_strfreev( Larrbuf );
	
	return( TotalTracks );
}
// 
// 
void CdCddb_cdparanoia_get_sectors ( gchar *buffer )
{
	gchar	**Larrbuf = NULL;
	gint	piste;
	gchar	*ptr = NULL;
	gint	Track = 0;
	
	Larrbuf = g_strsplit( buffer, "\n", 0 );
	for( piste = 0; Larrbuf[ piste ]; piste ++ ) {
		if( strstr( Larrbuf[ piste ], "TOTAL" )) {
			ptr = Larrbuf[ piste ];
			while( *ptr && *ptr != ' ' ) ptr ++;
			while( *ptr && *ptr == ' ' ) ptr ++;
			if( *ptr ) {
				VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = atoi( ptr );
			}
			break;
		}
		
		if( NULL != ( ptr = strchr( Larrbuf[ piste ], ']' ))) {
			ptr ++;
			while( *ptr && *ptr == ' ' ) ptr ++;
			if( *ptr )
				VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = atoi( ptr );
				Track ++;
		}
	}
	g_strfreev( Larrbuf );
}
// 
// GET TOTAL TRACKS WITH DATA_TRACK IF EXIST
// GET SECTORS
// 
void CdCddb_disc_info( CdIo_t *p_cdio )
{
	track_t		i_first_track = cdio_get_first_track_num( p_cdio );
	track_t		i_tracks      = cdio_get_num_tracks( p_cdio );
	track_t		i_last_track  = i_first_track + i_tracks;
	lba_t		SectorMoins   = cdio_get_track_lba( p_cdio, i_first_track );
	lba_t		Sector;
	gint		Track = 0;
	  
	PRINT_FUNC_LF();
	
	g_print( "\ttotal_tracks with cdio  = %d\n", i_last_track -1 );
	
	// GET TOTAL TRACKS WITH DATA_TRACK IF EXIST
	VarCddb . TotalTracks = i_last_track -1;
	
	// ALLOCATE DATAS SECTORS
	VarCddb . VarStructCddb = ( VAR_STRUCT_CDDB * ) g_malloc0( sizeof( VAR_STRUCT_CDDB ) * ( VarCddb . TotalTracks + 6 ));

	// INIT DATAS SECTORS
	for( Track = 0 ; Track < VarCddb . TotalTracks + 4; Track++ ) {
		VarCddb . VarStructCddb [ Track ] . Duree   = NULL;
		VarCddb . VarStructCddb [ Track ] . Artist  = NULL;
		VarCddb . VarStructCddb [ Track ] . Title   = NULL;
		VarCddb . VarStructCddb [ Track ] . Sector = -1;
		VarCddb . VarStructCddb [ Track ] . SectorCdParanoia = -1;
	}
	
	SectorMoins = cdio_get_track_lba( p_cdio, i_first_track );
	
	for ( ; i_first_track <= i_last_track; i_first_track++ ) {

		Sector = cdio_get_track_lba( p_cdio, i_first_track );
		VarCddb . VarStructCddb [ i_first_track -1 ] . Sector = ( Sector - SectorMoins );
		
		// g_print ("%02d  ", i_first_track );
		// g_print ("SECTOR = %d   ", Sector - SectorMoins );
		// g_print ( "\n" );
	}
}
// 
// 
gboolean CdCddb_get_from_cdio( void )
{
	CdIo_t		*p_cdio = NULL;
	discmode_t	cd_discmode;
	
	// PRINT_FUNC_LF();
	
	p_cdio = cdio_open( VarCD.NameCD_Device, DRIVER_DEVICE );	// DRIVER_DEVICE  DRIVER_UNKNOWN
	cd_discmode  = cdio_get_discmode(p_cdio);
	if( cd_discmode == CDIO_DISC_MODE_CD_DA ) {
		g_print("discmode2str = %s\n", discmode2str[ cd_discmode ] );
	} else {
		g_print( "\tCouldn't find CD\n" );
		return( FALSE );
	}
	
	if (NULL == p_cdio) {
		g_print("\tCouldn't find CD\n");
		return( FALSE );
	} else {
		CdCddb_disc_info( p_cdio );
		cdio_destroy( p_cdio );
	}
	
	return( TRUE );
}

#define CDDB_RET_OK		0
#define CDDB_RET_NO_CD		1
#define CDDB_RET_NO_CONNECT	2
#define CDDB_RET_NO_BASE	3
#define CDDB_RET_NO_ALLOC	4

// RETURN
// 	0  OK
// 	1  COULDN'T FIND CD
//	2  PAS DE CONNEXION WEB
//	3  PAS DE BASE POUR CE DISQUE
// 	4  PROBLEME D'ALLOCATION MEMOIRE
// 
gint CdCddb_read_datas (void)
{
	cddb_conn_t	*p_cddb = NULL;
   	cddb_disc_t	*p_disc = NULL;
   	gint		Track;
	gint		NbrMatchsFound;
	gint		min;
	gint		len;
	gint		RetValue = CDDB_RET_OK;
	GString		*gstr = NULL;
	gint		TotalTracks;
	
	// PRINT_FUNC_LF();
	
	// REMOVE VarCddb
	CdCddb_remove_struct();
	
	// GET TOTAL TRACKS WITH DATA_TRACK IF EXIST
	// GET SECTORS
	if( FALSE == CdCddb_get_from_cdio() ) {
		g_print( "COULDN'T FIND CD\n" );
 		strcpy( VarCddb . Message, _("Couldn't find CD") );
		RetValue = CDDB_RET_NO_CD;
		return( RetValue );
	}
	
	// GET REAL TRACKS AND GET REAL SECTORS FROM CDPARANOIA
	gstr = GetInfo_cdparanoia (VarCD.NameCD_Device);
	g_print("gstr->str=\n%s", gstr->str);
	if( ( TotalTracks = CdCddb_cdparanoia_get_total_tracks( gstr->str ) ) > -1 ) {
		CdCddb_cdparanoia_get_sectors( gstr->str );
	}
	g_string_free (gstr, TRUE);
	gstr = NULL;
	
	g_print( "total_tracks with cdparanoia = %d\n", TotalTracks );
	
	// CALL LIBCDDB
	if( NULL != ( p_cddb = cddb_new() ) ) {

		cddb_set_server_name( p_cddb, "freedb.org" );
		
		g_print( "-------------------------------------------------\n" );
		g_print( "SET ...\n" );
		cddb_set_server_port( p_cddb, 888 );
		g_print( "\tcddb_set_server_port: OK\n" );
		cddb_cache_disable( p_cddb );
		g_print( "\tcddb_cache_disable: OK\n" );
		cddb_set_timeout( p_cddb, 10 );
		g_print( "\tcddb_set_timeout: OK\n" );
		cddb_http_disable( p_cddb );
		g_print( "\tcddb_http_disable: OK\n" );
	
		if( NULL != ( p_disc = cddb_disc_new() ) ) {
			
			int64_t i_length = 0;
			
			g_print( "RECHERCHE CDDB\n" );
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				cddb_track_t *t = cddb_track_new();
				cddb_track_set_frame_offset( t, VarCddb . VarStructCddb [ Track ] . Sector );
				cddb_disc_add_track( p_disc, t );
				const int64_t i_size = ( VarCddb . VarStructCddb [ Track +1 ] . Sector - VarCddb . VarStructCddb [ Track ] . Sector ) *(int64_t)CDDA_DATA_SIZE;
				i_length += 1000000 * i_size / 44100 / 4;
			}
			g_print( "\tframe + track: OK\n" );
			cddb_disc_set_length( p_disc, (int)( i_length / 1000000 ) );
			cddb_disc_calc_discid( p_disc );
			g_print( "\tlength + discid: OK\n" );
			
			g_print( "-------------------------------------------------\n" );
			NbrMatchsFound = cddb_query( p_cddb, p_disc );
			g_print( "cddb_query()   = %d\n", NbrMatchsFound );
			g_print( "NbrMatchsFound = %d\n", NbrMatchsFound );
			if( NbrMatchsFound < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				RetValue = CDDB_RET_NO_CONNECT;
			}
			
			if( NbrMatchsFound < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				RetValue = CDDB_RET_NO_CONNECT;
			}
			else if( NbrMatchsFound == 0 ) {
				g_print( "PAS DE BASE CDDB POUR CE DISQUE\n" );
 				strcpy( VarCddb . Message, _("Pas de base CDDB pour ce disque") );
				RetValue = CDDB_RET_NO_BASE;
			}
			else if( NbrMatchsFound > 1 ) {
				g_print( "found %d matches in CDDB. Using first one.\n", NbrMatchsFound );
			}
			
			if( cddb_read( p_cddb, p_disc ) < 0 ) {
				g_print( "LA LIAISON INTERNET EST ABSENTE\n" );
 				strcpy( VarCddb . Message, _("La liaison internet est absente") );
				RetValue = CDDB_RET_NO_CONNECT;
			}

			g_print( "-------------------------------------------------\n" );
			if( CDDB_RET_NO_CONNECT != RetValue && CDDB_RET_NO_BASE != RetValue )
				VarCddb . TitleCD = g_strdup( cddb_disc_get_title( p_disc ));
			else	VarCddb . TitleCD = g_strdup( "TitleCD" );
			g_print( "TITLE  = %s\n", VarCddb . TitleCD);
			
			if( CDDB_RET_NO_CONNECT != RetValue && CDDB_RET_NO_BASE != RetValue )
				VarCddb . Artiste = g_strdup( cddb_disc_get_artist( p_disc ));
			else	VarCddb . Artiste = g_strdup( " " );
			g_print( "ARTIST = %s\n", VarCddb . Artiste);
			
			if( CDDB_RET_NO_CONNECT != RetValue && CDDB_RET_NO_BASE != RetValue )
				VarCddb . NumYear = cddb_disc_get_year( p_disc );
			else	VarCddb . NumYear = 1962;
			VarCddb . StrYear = g_strdup_printf( "%d", VarCddb . NumYear );
			g_print( "YEAR   = %d\n", VarCddb . NumYear);
			
			// g_sprintf (CdToc->Str_ID_CD, "%08lx", CdToc->Num_ID_CD);
			if( CDDB_RET_NO_CONNECT != RetValue && CDDB_RET_NO_BASE != RetValue )
				g_print( "DISCID = %x\n", cddb_disc_get_discid( p_disc ));
			else	g_print( "DISCID = 0\n");
			
			len = VarCddb . VarStructCddb [ TotalTracks ] . SectorCdParanoia - VarCddb . VarStructCddb [ 0 ] . SectorCdParanoia;
			VarCddb . StrDureeCd = g_strdup_printf( "%02d:%02d", ( len / 60 ) / 75,  ( len / 75 ) % 60 );
			g_print( "DUREE  = %s\n", VarCddb . StrDureeCd );
			g_print( "-------------------------------------------------\n" );
			len = 0;
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				
				g_print( "%02d  ", Track +1 );
								
				// CONNECT IS OK
				if( CDDB_RET_NO_CONNECT != RetValue && CDDB_RET_NO_BASE != RetValue ) {
					
					cddb_track_t *t = cddb_disc_get_track( p_disc, Track );
				
					if( NULL == t ) continue;
				
					// ARTISTE
					VarCddb . VarStructCddb [ Track ] . Artist = g_strdup( cddb_track_get_artist( t ));
					g_print( "%s\t", VarCddb . VarStructCddb [ Track ] . Artist);
				
					// TITRE PISTE
					VarCddb . VarStructCddb [ Track ] . Title = g_strdup( cddb_track_get_title( t ));
					g_print( "%s       ", VarCddb . VarStructCddb [ Track ] . Title);
					
				}
				// NO CONNECT
				else {
					// ARTISTE
					VarCddb . VarStructCddb [ Track ] . Artist = g_strdup( "Artist" );
					g_print( "%s\t", VarCddb . VarStructCddb [ Track ] . Artist);
				
					// TITRE PISTE
					VarCddb . VarStructCddb [ Track ] . Title = g_strdup( "Title" );
					g_print( "%s", VarCddb . VarStructCddb [ Track ] . Title);
				}
				
				g_print( "\n" );
			}
			
			g_print( "-------------------------------------------------\n" );
			g_print( "cddb_disc_get_category_str() = %s\n", cddb_disc_get_category_str( p_disc ));
			g_print( "cddb_disc_get_category()     = %d\n", cddb_disc_get_category( p_disc ));

			// BASE ADJUSTMENT FROM CDPARANOIA
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				VarCddb . VarStructCddb [ Track ] . Sector = VarCddb . VarStructCddb [ Track ] . SectorCdParanoia;
			}
			VarCddb . TotalTracks = TotalTracks;
			g_print( "RECTIF FOR : DataTrack\n" );
			g_print( "\tVarCddb . TotalTracks       = %d\n",  VarCddb . TotalTracks );
			g_print( "\tTotalTracks with cdparanoia = %d\n\n",  TotalTracks );
			for( Track = 0; Track < VarCddb . TotalTracks; Track++ ) {
				 // DUREE D'ECOUTE DES PISTES
				min = VarCddb . VarStructCddb [ Track +1 ] . Sector - VarCddb . VarStructCddb [ Track ] . Sector;
				VarCddb . VarStructCddb [ Track ] . Duree = g_strdup_printf( "%02d:%02d", ( min / 60 ) / 75, ( min / 75 ) % 60 );
			}
						
			cddb_disc_destroy( p_disc );
		}
		cddb_destroy( p_cddb );
	}

	return( RetValue );
}
// P_ETAT = 
// 	->0  OK
// 	  1  PROBLEME D'ALLOCATION MEMOIRE
// 	  2  CD NON AUDIO OU CD ABSENT DU LECTEUR
// 	  3  LE CD DANS LECTEUR N'EST PAS UN CD-AUDIO
//	->4  PAS DE CONNEXION WEB
//	->5  PAS DE BASE POUR CE DISQUE
//	  6  ERREUR DE LECTURE DEPUIS LE CD
//	  7  CD ABSENT DU LECTEUR
// 
void CdCddb_set_toc_to_glist_audio (gint p_etat)
{
	gint		NumList;
	GString 	*gstr_buffer_title = g_string_new (NULL);
	GString 	*gstr_buffer_title_time = g_string_new (NULL);
	GString 	*gstr_buffer_artist_title_time = g_string_new (NULL);
	
	// PRINT_FUNC_LF();

	// REMOVE DATAS entete
	poche_remove_view();
			
	for( NumList = 0; NumList < VarCddb . TotalTracks; NumList++ ) {
		
		g_print( "TITLE_CD: %s  ", VarCddb . TitleCD );
		g_print( "TRACK: %d  ", NumList + 1 );
		g_print( "DUREE: %s  ", VarCddb . VarStructCddb [ NumList ] . Duree );
		g_print( "ARTIST: %s  ", VarCddb . VarStructCddb [ NumList ] . Artist );
		g_print( "TITLE: %s  ", VarCddb . VarStructCddb [ NumList ] . Title );
		g_print( "YEAR: %d  ", VarCddb . NumYear );
		
		// OK
		if( CDDB_RET_OK == p_etat ) {
			
			
			view.TitleCD = g_strdup( VarCddb.TitleCD );
			// AUCUN

			// TITRE
			g_string_append_printf( gstr_buffer_title, "%s\n", VarCddb . VarStructCddb [ NumList ] . Title );

			// TITRE - TEMPS
			g_string_append_printf( gstr_buffer_title_time, "%s  ][  ", VarCddb . VarStructCddb [ NumList ] . Title );
			g_string_append_printf( gstr_buffer_title_time, "%s\n", VarCddb . VarStructCddb [ NumList ] . Duree );

			// ATISTE - TITRE - TEMPS
			g_string_append_printf( gstr_buffer_artist_title_time, "%s  -  ", VarCddb . VarStructCddb [ NumList ] . Artist );
			g_string_append_printf( gstr_buffer_artist_title_time, "%s  ][  ", VarCddb . VarStructCddb [ NumList ] . Title );
			g_string_append_printf( gstr_buffer_artist_title_time, "%s\n", VarCddb . VarStructCddb [ NumList ] . Duree );
		}
		
		// PAS DE CONNEXION WEB
		else if( CDDB_RET_NO_CONNECT == p_etat ) {
			
		}
		
		// PAS DE BASE POUR CE DISQUE
		else if( CDDB_RET_NO_BASE == p_etat ) {
				
		}
		g_print( "\n" );

	}
	if( CDDB_RET_OK == p_etat || CDDB_RET_NO_CONNECT == p_etat || CDDB_RET_NO_BASE == p_etat ) {
		
		view.Buffer_none = g_strdup( " " );
		
		view.Buffer_title = g_strdup( gstr_buffer_title->str );
		g_string_free( gstr_buffer_title, TRUE );
		
		view.Buffer_title_time = g_strdup( gstr_buffer_title_time->str );
		g_string_free( gstr_buffer_title_time, TRUE );
		
		view.Buffer_artist_title_time = g_strdup( gstr_buffer_artist_title_time->str );
		g_string_free( gstr_buffer_artist_title_time, TRUE );
	}
}
// 
// 
gboolean CdCddb_read_toc( void )
{
	gint	Etat;
	
	// 
	Etat = CdCddb_read_datas();
	
	// TODO: DEBUG: manuker: http://forum.ubuntu-fr.org/viewtopic.php?pid=4631891#p4631891
	g_print("Return after call CdCddb_read_datas(): ");
	switch( Etat ) {
	case CDDB_RET_OK :		g_print( "CDDB_RET_OK\n" );		break;
	case CDDB_RET_NO_CD:		g_print( "CDDB_RET_NO_CD\n" );		break;
	case CDDB_RET_NO_CONNECT :	g_print( "CDDB_RET_NO_CONNECT\n" );	break;
	case CDDB_RET_NO_BASE :		g_print( "CDDB_RET_NO_BASE\n" );	break;
	case CDDB_RET_NO_ALLOC :	g_print( "CDDB_RET_NO_ALLOC\n" );	break;
	}
	
	// 
	CdCddb_set_toc_to_glist_audio( Etat );
		
	if( CDDB_RET_OK == Etat || CDDB_RET_NO_CONNECT == Etat || CDDB_RET_NO_BASE == Etat ) {
		return( TRUE );
	}
	return ( FALSE );
}
// 
// 
void *CdCddb_thread (void *data)
{
	VarCD.bool_thread_read_toc_cd = TRUE;
	
	VarCD.Bool_Read_Infos_cd = CdCddb_read_toc();

	VarCD.bool_thread_read_toc_cd = FALSE;
	pthread_exit (0);
}
/* Mise a jour des informations du cd-audio
*  --
*  entree :
*      gpointer data :
*  retour :
*      TRUE
*/
static gint CdCddb_timeout (gpointer data)
{
	if (VarCD.bool_thread_read_toc_cd == FALSE) {

		// FIN du timeout Update_cdaudio_do_read_toc_cd
		gtk_timeout_remove (VarCD.handler_timeout_read_toc_cd);
		
		if( VarCD.Bool_Read_Infos_cd == TRUE ) {
			switch( VarCD.Choice ) {
			case _CDDB_BUFFER_NONE_ :
				pochetxt_set_text_from_cd( view.Buffer_none, VarCD.Choice );
				break;
			case _CDDB_BUFFER_TITLE_ : 
				pochetxt_set_text_from_cd( view.Buffer_title, VarCD.Choice );
				break;
			case _CDDB_BUFFER_TITLE_TIME_ : 
				pochetxt_set_text_from_cd( view.Buffer_title_time, VarCD.Choice );
				break;
			case _CDDB_BUFFER_ARTIST_TITLE_TIME_ : 
				pochetxt_set_text_from_cd( view.Buffer_artist_title_time, VarCD.Choice );
				break;
			}
		}
		else {
			pochetxt_set_text_from_cd( " ", VarCD.Choice );
		}
		VarCD.bool_timeout_read_toc_cd = FALSE;
	
		WindScan_close ();
		
		StatusBar_puts( VarCddb.Message );
		
		// REMOVE VarCddb
		CdCddb_remove_struct();
		
		poche_set_flag_buttons ();

	}
	return (TRUE);
}
/* Lecture du catalogue du cd-audio
*  --
*  entree : -
*  retour : -
*/
void CdCddb_reffresh_list_cd( gint p_Choice )
{
	pthread_t nmr_tid;
	
	if( NULL == view.AdrComboBoxListCD ) return;
	
	VarCD.Choice = p_Choice;

	WindScan_open ("Scan CD", WINDSCAN_PULSE);
	WindScan_set_label ("<b><i>Scanning CD ...</i></b>");
	WindScan_set_pulse ();

	VarCD.NameCD_Device = scan_get_text_combo_cd( view.AdrComboBoxListCD );
	
	// recherche CDDB is TRUE
	CdCddb_set_bool_cancel_cddb (FALSE);

	// ACTIVATION de l'update qui attends la fin du thread cd audio
	VarCD.bool_thread_read_toc_cd = TRUE;
	VarCD.bool_timeout_read_toc_cd = TRUE;
	VarCD.handler_timeout_read_toc_cd = gtk_timeout_add (50, CdCddb_timeout, 0);
	pthread_create (&nmr_tid, NULL ,(void *)CdCddb_thread, (void *)NULL);
}



void CdCddb_set_combobox_choice( gint p_activate )
{
	if( p_activate > -1 ) {
		view.BoolAccessChanged = FALSE;
		gtk_combo_box_set_active( GTK_COMBO_BOX (view.AdrComboBoxChoice), p_activate );
		view.BoolAccessChanged = TRUE;
	}
}

gchar *CdCddb_get_title_cd( void )
{
	return( view.TitleCD );
}



